namespace Cable.Adapter.Irc {
    Maki client;
}

class Cable.Adapter.Irc.Session : Object, Port.Irc.Session {

    Gee.List <Server> servers;

    string[] server_names {
        owned get {
            try {
                return client.servers ();
            } catch (GLib.IOError e) {
                warning (e.message);
            }

            return {};
        }
    }

    string[] known_server_names {
        owned get {
            try {
                return client.server_list ("", "");
            } catch (GLib.IOError e) {
                warning (e.message);
            }

            return {};
        }
    }
    
    public Session () {
        init ();
    }

    public void init () {
        if (client == null) {
            try {
                client = Bus.get_proxy_sync (GLib.BusType.SESSION, "de.ikkoku.sushi", "/de/ikkoku/sushi");
            } catch (GLib.IOError error) {
                warning (error.message);
            }

            debug ("Successfully connected to Sushi dbus session.\n");
        }

        servers = new Gee.ArrayList <Server> ();

        // TODO do something more intelligent here
        foreach (var server_name in known_server_names) {
            var server = new Server.from_name (server_name);
            foreach (var channel in server.channel_names)
                server.send_part (channel);
            servers.add (server);
        }
        
    }

    public Port.Irc.Server get_or_create_server (string address, string nick) {
        return (get_server (address, nick) ??
               create_server (address, nick)) as Port.Irc.Server;
    }

    Server? get_server (string address, string nick) {
        foreach (var server in servers)
            if (server.address == address && server.nick == nick)
                return server as Server;
        return (Server) null;
    }

    Server create_server (string address, string nick) {
        var server = new Server (address, nick);
        servers.add (server);
        return server;
    }
}
