namespace Cable.Port.Irc {

    public interface Session : Object {

        public abstract void init ();
        public abstract Server get_or_create_server (string address, string nick);
    }

    public interface Server : Object {

        public abstract Channel get_or_create_channel (string channel_name);

        public abstract string address { owned get; }
        public abstract string nick { owned get; }
        public abstract string name { get; set; }
        public abstract bool away { get; }
        public abstract bool connected { get; }
        //public abstract string[] channel_names { owned get; }
        //public abstract string[] supported_prefixes { owned get; }
        //public abstract string supported_channel_types { owned get; }
        //public abstract string[] ignores { owned get; }

        public signal void event_away ();
        public signal void event_away_message (string nick, string message);
        public signal void event_back ();
        public signal void event_connected ();
        public signal void event_error (string domain, string reason, string[] arguments);
        public signal void event_invite (string from, string channel, string who);
        public signal void event_list (string channel, uint64 users, string topic);
        public signal void event_message (string from, string target, string message);
        public signal void event_mode (string from, string target, string mode, string parameter);
        public signal void event_motd (string message);
        public signal void event_nick (string from, string new_nick);
        public signal void event_no_such (string target, string type);
        public signal void event_notice (string from, string target, string message);
        public signal void event_oper ();
        public signal void event_quit (string from, string message);
        public signal void event_user_away (string from, bool away);
        public signal void event_whois (string nick, string message);

        public abstract void send_away (string? message = null);
        public abstract void send_back ();
        public abstract void send_connect ();
        public abstract void send_ignore (string pattern);
        public abstract void send_message (string target, string message);
        //public abstract void send_mode (string target, string mode);
        public abstract void send_nick (string nick);
        //public abstract void send_nickserv ();
        public abstract void send_notice (string target, string message);
        //public abstract void send_oper (string name, string password);
        public abstract void send_part (string channel, string? message = null);
        public abstract void send_quit (string? message = null);
        public abstract void send_raw (string command);
        public abstract void send_unignore (string pattern);
        //public abstract void send_who (string mask, bool operators_only);
        //public abstract void send_whois (string mask);

        public abstract bool is_me (string nick);
    }

    public interface Channel : Object {

        public abstract string name { get; set; }
        public abstract string topic { get; }
        public abstract string[] nicks { get; }
        public abstract string[] prefixes { get; }
        public abstract bool joined { get; }

        public signal void event_action (string from, string message);
        public signal void event_invite (string from, string who);
        public signal void event_join (string from);
        public signal void event_kick (string from, string who, string message);
        public signal void event_message (string from, string message);
        public signal void event_part (string from, string message);
        public signal void event_topic (string from, string topic);
        public signal void event_banlist (string mask, string who, int64 when);
        public signal void event_cannot_join (string reason);
        public signal void event_list (int64 users, string topic);
        public signal void event_names (string[] nicks, string[] prefixes);
        public signal void event_no_such ();

        public abstract void send_action (string message);
        //public abstract void send_invite (string who);
        public abstract void send_join (string key = "");
        public abstract void send_kick (string who, string message);
        //public abstract void send_list ();
        public abstract void send_message (string message);
        public abstract void send_names ();
        public abstract void send_part (string? message = null);
        public abstract void send_topic (string topic = "");

        //public abstract string user_channel_mode (string nick);
        //public abstract string user_channel_prefix (string nick);
    }
}
